@@ -1,7 +1,10 @@
 package Pod::Coverage::Imager;
+use 5.006;
 use strict;
 use base 'Pod::Coverage';
 
+our $VERSION = "1.000";
+
 sub _get_pods {
   my $self = shift;
 
@@ -26,4 +29,20 @@ sub _get_pods {
   return $pod->{identifiers} || [];
 }
 
+sub _get_syms {
+  my ($self, $package) = @_;
+
+  if ($self->{module}) {
+    eval "require $self->{module}";
+    return if $@;
+
+    # fake out require
+    (my $file = $package) =~ s(::)(/)g;
+    $file .= ".pm";
+    $INC{$file} = 1;
+  }
+
+  return $self->SUPER::_get_syms($package);
+}
+
 1;
